# -*- coding: utf-8 -*-

"""
The :mod:`pipesnake.transformers` module implements several transformers.

Here is a short list of the main modules:

:mod:`pipesnake.transformers.combiner`
:mod:`pipesnake.transformers.converter`
:mod:`pipesnake.transformers.deeplearning`
:mod:`pipesnake.transformers.dropper`
:mod:`pipesnake.transformers.financial`
:mod:`pipesnake.transformers.imputer`
:mod:`pipesnake.transformers.misc`
:mod:`pipesnake.transformers.scaler`
:mod:`pipesnake.transformers.selector`
:mod:`pipesnake.transformers.stats`
"""
