# -*- coding: utf-8 -*-
"""Metadata for pipesnake"""

# The package name, which is also the "UNIX name" for the project.
package = 'pipesnake'
project = 'Pipesnake'
project_no_spaces = project.replace(' ', '')
version = '0.1'
description = 'Feature extractor and data processing pipelines for Pandas inspired by Scikit-Learn.'
authors = ['Pierluigi Failla']
authors_string = ', '.join(authors)
emails = ['{name}.{surname}@gmail.com']
license = 'MIT'
copyright = '2018 ' + authors_string
url = 'https://github.com/pierluigi-failla/pipesnake'
